@@ -1,8 +1,8 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
+  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2014 The PHP Group                                |
+  | Copyright (c) 1997-2016 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -43,7 +43,12 @@
 #define PDO_DBG_ERR(msg) do { if (dbg_skip_trace == FALSE) PDO_MYSQL_G(dbg)->m->log(PDO_MYSQL_G(dbg), __LINE__, __FILE__, -1, "error: ", (msg)); } while (0)
 #define PDO_DBG_INF_FMT(...) do { if (dbg_skip_trace == FALSE) PDO_MYSQL_G(dbg)->m->log_va(PDO_MYSQL_G(dbg), __LINE__, __FILE__, -1, "info : ", __VA_ARGS__); } while (0)
 #define PDO_DBG_ERR_FMT(...) do { if (dbg_skip_trace == FALSE) PDO_MYSQL_G(dbg)->m->log_va(PDO_MYSQL_G(dbg), __LINE__, __FILE__, -1, "error: ", __VA_ARGS__); } while (0)
-#define PDO_DBG_ENTER(func_name) zend_bool dbg_skip_trace = TRUE; if (PDO_MYSQL_G(dbg)) dbg_skip_trace = !PDO_MYSQL_G(dbg)->m->func_enter(PDO_MYSQL_G(dbg), __LINE__, __FILE__, func_name, strlen(func_name));
+#define PDO_DBG_ENTER(func_name) \
+	zend_bool dbg_skip_trace = TRUE; \
+	((void) dbg_skip_trace); \
+	if (PDO_MYSQL_G(dbg)) \
+		dbg_skip_trace = !PDO_MYSQL_G(dbg)->m->func_enter(PDO_MYSQL_G(dbg), __LINE__, __FILE__, func_name, strlen(func_name));
+
 #define PDO_DBG_RETURN(value)	do { if (PDO_MYSQL_G(dbg)) PDO_MYSQL_G(dbg)->m->func_leave(PDO_MYSQL_G(dbg), __LINE__, __FILE__, 0); return (value); } while (0)
 #define PDO_DBG_VOID_RETURN		do { if (PDO_MYSQL_G(dbg)) PDO_MYSQL_G(dbg)->m->func_leave(PDO_MYSQL_G(dbg), __LINE__, __FILE__, 0); return; } while (0)
 
@@ -81,11 +86,10 @@ ZEND_BEGIN_MODULE_GLOBALS(pdo_mysql)
 ZEND_END_MODULE_GLOBALS(pdo_mysql)
 
 ZEND_EXTERN_MODULE_GLOBALS(pdo_mysql)
+#define PDO_MYSQL_G(v) ZEND_MODULE_GLOBALS_ACCESSOR(pdo_mysql, v)
 
-#ifdef ZTS
-#define PDO_MYSQL_G(v) TSRMG(pdo_mysql_globals_id, zend_pdo_mysql_globals *, v)
-#else
-#define PDO_MYSQL_G(v) (pdo_mysql_globals.v)
+#if defined(ZTS) && defined(COMPILE_DL_PDO_MYSQL)
+ZEND_TSRMLS_CACHE_EXTERN()
 #endif
 
 
@@ -104,9 +108,9 @@ typedef struct {
 	unsigned buffered:1;
 	unsigned emulate_prepare:1;
 	unsigned fetch_table_names:1;
-	unsigned _reserved:31;	
+	unsigned _reserved:31;
 #if !PDO_USE_MYSQLND
-	unsigned long max_buffer_size;
+	zend_ulong max_buffer_size;
 #endif
 
 	pdo_mysql_error_info einfo;
@@ -118,38 +122,38 @@ typedef struct {
 
 typedef struct {
 	pdo_mysql_db_handle 	*H;
-	MYSQL_RES		*result;
-	const MYSQL_FIELD	*fields;
-	MYSQL_ROW		current_data;
+	MYSQL_RES				*result;
+	const MYSQL_FIELD		*fields;
+	MYSQL_ROW				current_data;
 #if PDO_USE_MYSQLND
-	unsigned long		*current_lengths;
+	zend_ulong		*current_lengths;
 #else
-	long			*current_lengths;
+	zend_long		*current_lengths;
 #endif
-	pdo_mysql_error_info einfo;
+	pdo_mysql_error_info 	einfo;
 #if PDO_USE_MYSQLND
-	MYSQLND_STMT 		*stmt;
+	MYSQLND_STMT 			*stmt;
 #else
-	MYSQL_STMT		*stmt;
-#endif	
-	int num_params;
+	MYSQL_STMT				*stmt;
+#endif
+	int 					num_params;
 	PDO_MYSQL_PARAM_BIND	*params;
 #ifndef PDO_USE_MYSQLND
-	my_bool			*in_null;
-	unsigned long		*in_length;
+	my_bool					*in_null;
+	zend_ulong			*in_length;
 #endif
 	PDO_MYSQL_PARAM_BIND	*bound_result;
-	my_bool			*out_null;
-	unsigned long		*out_length;
-	unsigned int		params_given;
-	unsigned		max_length:1;
+	my_bool					*out_null;
+	zend_ulong			*out_length;
+	unsigned int			params_given;
+	unsigned				max_length:1;
 } pdo_mysql_stmt;
 
 extern pdo_driver_t pdo_mysql_driver;
 
-extern int _pdo_mysql_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *file, int line TSRMLS_DC);
-#define pdo_mysql_error(s) _pdo_mysql_error(s, NULL, __FILE__, __LINE__ TSRMLS_CC)
-#define pdo_mysql_error_stmt(s) _pdo_mysql_error(stmt->dbh, stmt, __FILE__, __LINE__ TSRMLS_CC)
+extern int _pdo_mysql_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *file, int line);
+#define pdo_mysql_error(s) _pdo_mysql_error(s, NULL, __FILE__, __LINE__)
+#define pdo_mysql_error_stmt(s) _pdo_mysql_error(stmt->dbh, stmt, __FILE__, __LINE__)
 
 extern struct pdo_stmt_methods mysql_stmt_methods;
 
